@@ -18,7 +18,7 @@ import logging
 import re
 import sys
 from datetime import datetime
-from typing import Any as _Any, Dict, Mapping as _Mapping, Tuple, Union
+from typing import Dict, Tuple, Union, Any as _Any
 import arrow
 from assemblyline.common.net import is_valid_domain, is_valid_ip
 from dateutil.tz import tzutc
@@ -40,7 +40,7 @@ BANNED_FIELDS = {"id", "__access_grp1__", "__access_lvl__", "__access_req__", "_
 DATEFORMAT = "%Y-%m-%dT%H:%M:%S.%fZ"
 FIELD_SANITIZER = re.compile("^[a-z][a-z0-9_]*$")
 FLATTENED_OBJECT_SANITIZER = re.compile("^[a-z][a-z0-9_.]*$")
-NOT_INDEXED_SANITIZER = re.compile("^[A-Za-z0-9_ ]*$")
+NOT_INDEXED_SANITIZER = re.compile("^[A-Za-z0-9_ -]*$")
 UTC_TZ = tzutc()
 
 DOMAIN_REGEX = r"(?:(?:[A-Za-z0-9\u00a1-\uffff][A-Za-z0-9\u00a1-\uffff_-]{0,62})?[A-Za-z0-9\u00a1-\uffff]\.)+" \
@@ -821,7 +821,7 @@ class Optional(_Field):
 
 class Model:
     @classmethod
-    def fields(cls, skip_mappings=False) -> _Mapping[str, _Field]:
+    def fields(cls, skip_mappings=False) -> dict[str, _Field]:
         """
         Describe the elements of the model.
 
@@ -875,7 +875,7 @@ class Model:
         return out
 
     @classmethod
-    def flat_fields(cls, show_compound=False, skip_mappings=False) -> _Mapping[str, _Field]:
+    def flat_fields(cls, show_compound=False, skip_mappings=False) -> dict[str, _Field]:
         """
         Describe the elements of the model.
 
@@ -920,9 +920,10 @@ class Model:
                 return f"[{name}]({module_path}/#{name.lower()})", field_class.child_type
             elif field_class.__class__ in [Mapping, List]:
                 child_type, child_class = field_class.child_type.__class__.__name__, field_class.child_type.__class__
-                if field_class.child_type.__class__ == Compound:
+                if field_class.child_type.__class__ in [Compound, Mapping, List]:
                     child_type, child_class = get_type(field_class.child_type)
-                return f"{field_class.__class__.__name__} [{child_type}]", child_class
+                prefix_insert = "String, " if field_class.__class__ == Mapping else ''
+                return f"{field_class.__class__.__name__} [{prefix_insert}{child_type}]", child_class
             elif field_class.__class__.__name__ == 'type':
                 return field_class.__name__, None
 
@@ -933,8 +934,9 @@ class Model:
             field_type, field_class = get_type(info)
 
             # If the field is in fact a model class, generate markdown and append to list
-            if field_class and issubclass(field_class, Model) and field_class.__module__ == cls.__module__:
-                model_deps.append(field_class.markdown(toc_depth=toc_depth+1, defaults=info.default))
+            if field_class and issubclass(field_class, Model) and field_class.__module__ == cls.__module__ \
+                    and not any(dep[0] == field_class for dep in model_deps):
+                model_deps.append((field_class, field_class.markdown(toc_depth=toc_depth+1, defaults=info.default)))
 
             # Field description
             description = info.description
@@ -975,7 +977,7 @@ class Model:
 
         markdown_content += table + "\n\n"
         # Display model dependencies in order
-        for markdown in sorted(model_deps):
+        for _, markdown in sorted(model_deps, key=lambda x: x[1]):
             markdown_content += markdown
 
         return markdown_content
@@ -1024,7 +1026,7 @@ class Model:
                              f"{', '.join(unused_keys)}")
         if unused_keys and ignore_extra_values and mask is None:
             logger.warning(
-                f"The following parameters where ignored from object "
+                f"The following parameters were ignored from object "
                 f"'{self.__class__.__name__}': {', '.join(unused_keys)}")
 
         # Pass each value through it's respective validator, and store it
