@@ -355,6 +355,8 @@ class Expiry(odm.Model):
     delete_storage = odm.Boolean(description="Should we also cleanup the file storage?")
     sleep_time = odm.Integer(description="Time, in seconds, to sleep in between each expiry run")
     workers = odm.Integer(description="Number of concurrent workers")
+    delete_workers = odm.Integer(description="Worker processes for file storage deletes.")
+    iteration_max_tasks = odm.Integer(description="How many query chunks get run per iteration.")
 
 
 DEFAULT_EXPIRY = {
@@ -362,7 +364,9 @@ DEFAULT_EXPIRY = {
     'delay': 0,
     'delete_storage': True,
     'sleep_time': 15,
-    'workers': 20
+    'workers': 20,
+    'delete_workers': 2,
+    'iteration_max_tasks': 20,
 }
 
 
@@ -537,6 +541,46 @@ DEFAULT_SCALER = {
 }
 
 
+@odm.model(index=False, store=False)
+class VacuumSafelistItem(odm.Model):
+    name = odm.Keyword()
+    conditions = odm.Mapping(odm.Keyword())
+
+
+@odm.model(index=False, store=False)
+class Vacuum(odm.Model):
+    list_cache_directory: str = odm.Keyword()
+    worker_cache_directory: str = odm.Keyword()
+    data_directories: List[str] = odm.List(odm.Keyword())
+    file_directories: List[str] = odm.List(odm.Keyword())
+    assemblyline_user: str = odm.Keyword()
+    department_map_url = odm.Optional(odm.Keyword())
+    department_map_init = odm.Optional(odm.Keyword())
+    stream_map_url = odm.Optional(odm.Keyword())
+    stream_map_init = odm.Optional(odm.Keyword())
+    safelist = odm.List(odm.Compound(VacuumSafelistItem))
+    worker_threads: int = odm.Integer()
+    worker_rollover: int = odm.Integer()
+    minimum_classification: str = odm.Keyword()
+
+
+DEFAULT_VACUUM = dict(
+    list_cache_directory="/cache/",
+    worker_cache_directory="/memory/",
+    data_directories=[],
+    file_directories=[],
+    assemblyline_user="vacuum-service-account",
+    department_map_url=None,
+    department_map_init=None,
+    stream_map_url=None,
+    stream_map_init=None,
+    safelist=[],
+    worker_threads=50,
+    worker_rollover=1000,
+    minimum_classification='U',
+)
+
+
 @odm.model(index=False, store=False, description="Core Component Configuration")
 class Core(odm.Model):
     alerter: Alerter = odm.Compound(Alerter, default=DEFAULT_ALERTER, description="Configuration for Alerter")
@@ -548,6 +592,7 @@ class Core(odm.Model):
                                     description="Configuration for Metrics Collection")
     redis: Redis = odm.Compound(Redis, default=DEFAULT_REDIS, description="Configuration for Redis instances")
     scaler: Scaler = odm.Compound(Scaler, default=DEFAULT_SCALER, description="Configuration for Scaler")
+    vacuum: Vacuum = odm.Compound(Vacuum, default=DEFAULT_VACUUM, description="Configuration for Vacuum")
 
 
 DEFAULT_CORE = {
@@ -705,6 +750,7 @@ SERVICE_STAGES = [
 ]
 
 SAFELIST_HASH_TYPES = ['sha1', 'sha256', 'md5']
+REGISTRY_TYPES = ['docker', 'harbor']
 
 
 @odm.model(index=False, store=False, description="Service's Safelisting Configuration")
@@ -718,6 +764,14 @@ class ServiceSafelist(odm.Model):
                                            description="Should the Safelist service always run on extracted files?")
 
 
+@odm.model(index=False, store=False, description="Pre-Configured Registry Details for Services")
+class ServiceRegistry(odm.Model):
+    name: str = odm.Keyword(description="Name of container registry")
+    type: str = odm.Enum(values=REGISTRY_TYPES, default='docker', description="Type of container registry")
+    username: str = odm.Keyword(description="Username for container registry")
+    password: str = odm.Keyword(description="Password for container registry")
+
+
 @odm.model(index=False, store=False, description="Services Configuration")
 class Services(odm.Model):
     categories: List[str] = odm.List(odm.Keyword(), description="List of categories a service can be assigned to")
@@ -733,7 +787,7 @@ class Services(odm.Model):
     preferred_update_channel: str = odm.Keyword(description="Default update channel to be used for new services")
     allow_insecure_registry: bool = odm.Boolean(description="Allow fetching container images from insecure registries")
     preferred_registry_type: str = odm.Enum(
-        values=["docker", "harbor"],
+        values=REGISTRY_TYPES,
         default='docker',
         description="Global registry type to be used for fetching updates for a service (overridable by a service)")
     prefer_service_privileged: bool = odm.Boolean(
@@ -746,6 +800,8 @@ class Services(odm.Model):
         "At `0` (only for very small appliances/dev boxes), the service's CPU will be limited "
         "but no CPU will be reserved allowing for more flexible scheduling of containers.")
     safelist = odm.Compound(ServiceSafelist)
+    registries = odm.Optional(odm.List(odm.Compound(ServiceRegistry)),
+                              description="Global set of registries for services")
 
 
 DEFAULT_SERVICES = {
@@ -762,7 +818,8 @@ DEFAULT_SERVICES = {
         "enabled": True,
         "hash_types": ['sha1', 'sha256'],
         "enforce_safelist_service": False
-    }
+    },
+    "registries": []
 }
 
 
@@ -814,6 +871,7 @@ class AlertingMeta(odm.Model):
 
 DEFAULT_ALERTING_META = {
     'important': [
+        'original_source',
         'protocol',
         'subject',
         'submitted_url',
@@ -953,6 +1011,46 @@ DEFAULT_TAG_TYPES = {
 }
 
 
+@odm.model(index=False, store=False, description="A source entry for the sha256 downloader")
+class Sha256Source(odm.Model):
+    name: str = odm.Keyword(description="Name of the sha256 source")
+    classification = odm.Optional(
+        odm.ClassificationString(
+            description="Minimum classification applied to the downloaded "
+                        "files and required to know the existance of the source."))
+    data: str = odm.Optional(odm.Keyword(description="Data block sent during the URL call (Uses replace pattern)"))
+    failure_pattern: str = odm.Optional(odm.Keyword(
+        description="Pattern to find as a failure case when API return 200 OK on failures..."))
+    method: str = odm.Enum(values=['GET', 'POST'], default="GET", description="Method used to call the URL")
+    url: str = odm.Keyword(description="Url to fetch the file via SHA256 from (Uses replace pattern)")
+    replace_pattern: str = odm.Keyword(description="Pattern to replace in the URL with the SHA256")
+    headers: Dict[str, str] = odm.Mapping(odm.Keyword(), default={},
+                                          description="Headers used to connect to the URL")
+    proxies: Dict[str, str] = odm.Mapping(odm.Keyword(), default={},
+                                          description="Proxy used to connect to the URL")
+    verify: bool = odm.Boolean(default=True, description="Should the download function Verify SSL connections?")
+
+
+EXAMPLE_SHA256_SOURCE_VT = {
+    # This is an example on how this would work with VirusTotal
+    "name": "VirusTotal",
+    "url": r"https://www.virustotal.com/api/v3/files/{SHA256}/download",
+    "replace_pattern": r"{SHA256}",
+    "headers": {"x-apikey": "YOUR_KEY"},
+}
+
+EXAMPLE_SHA256_SOURCE_MB = {
+    # This is an example on how this would work with Malware Bazaar
+    "name": "Malware Bazaar",
+    "url": r"https://mb-api.abuse.ch/api/v1/",
+    "headers": {"Content-Type": "application/x-www-form-urlencoded"},
+    "data": r"query=get_file&sha256_hash={SHA256}",
+    "method": "POST",
+    "replace_pattern": r"{SHA256}",
+    "failure_pattern": '"query_status": "file_not_found"'
+}
+
+
 @odm.model(index=False, store=False,
            description="Default values for parameters for submissions that may be overridden on a per submission basis")
 class Submission(odm.Model):
@@ -965,6 +1063,9 @@ class Submission(odm.Model):
     max_file_size: int = odm.Integer(description="Maximum size for files submitted in the system")
     max_metadata_length: int = odm.Integer(description="Maximum length for each metadata values")
     max_temp_data_length: int = odm.Integer(description="Maximum length for each temporary data values")
+    sha256_sources: List[Sha256Source] = odm.List(
+        odm.Compound(Sha256Source),
+        default=[], description="List of external source to fetch file via their SHA256 hashes")
     tag_types = odm.Compound(TagTypes, default=DEFAULT_TAG_TYPES,
                              description="Tag types that show up in the submission summary")
 
@@ -978,6 +1079,7 @@ DEFAULT_SUBMISSION = {
     'max_file_size': 104857600,
     'max_metadata_length': 4096,
     'max_temp_data_length': 4096,
+    'sha256_sources': [],
     'tag_types': DEFAULT_TAG_TYPES
 }
 
